// superweather 2.1.1
// conceptualspace.net
// absolutely no warranty is expressed or implied

document.addEventListener("resume", onResume, false);

function onResume() {
	location.reload();
}

// show a loading message while the map renders
$(".timedisplay").show().html("loading...");

// get current time
// later we will round it back 10 mins because our RRAI radar data is only available in 10 min intervals
var now = new Date();

// click timer/listener to fade time display after z seconds of inactivity (we update lastclick on the nav button clicks)
var lastclick = now;
var display = 0;
setInterval(function() {
	var n = new Date();
	var z = n - lastclick;
	if(display == 1) {
		if (z > 8000) {
			$(".timedisplay").fadeOut(2000);
			display=0;
		}
	}
}, 5000);

// index to keep track of which time layer we're on for the nav buttons
var layerIndex = 1;

// functions to massage the date into ISO format for our WMS request
// wms expects hours and times in double digits, ie 09:05 vs 9:5. add a 0 when necessary
function pad(i) {
	if (i < 10) {
		i = '0' + i;
	}
	return i;
}

// lets wind our minutes back to the nearest 10
function roundMin(i) {
	if (i < 10) {
		i = '00';
	}
	else {
		// need to convert to string to extract first digit
		i = String(i);
		i = i.substr(0,1) + '0';
	}
	return i;
}

// round and convert to the ISO format for our environment canada WMS request
var maptime = now.getUTCFullYear() + '-' + pad(now.getUTCMonth() +1) + '-' + pad(now.getUTCDate()) + 'T' + pad(now.getUTCHours()) + ':' + roundMin(now.getUTCMinutes()) + ':00Z';

// take a time and roll it back by specified hours. used to get other wms layers
function newMaptime(time, hours) {
// check that its after midnight
	if (parseInt(time.substring(11,13)) >= hours) {
		// roll back an hour
		var prevHour = parseInt(time.substring(11,13))-hours;
		// prepend a 0 to the hour if necessary
		if (prevHour >= 10) {
			return time.substring(0,11) + String(prevHour) + time.substring(13);
		} else {
			return time.substring(0,11) + '0' + String(prevHour) + time.substring(13);
		}
	} else {
		// its midnight - need to roll back a day and x hours
		var prevDay = parseInt(time.substring(8,10))-1;
		// hour of prev day depends on current hour and hours we rolling back
		prevHour = 24 - hours + parseInt(time.substring(11,13));
		if (prevDay >= 10) {
			return time.substring(0,8) + String(prevDay) + 'T' + prevHour + time.substring(13);
		} else {
			return time.substring(0,8) + '0' + String(prevDay) + 'T' + prevHour + time.substring(13);
		}
	}
}


// convert from 24h UTC (for our wms request) to local am/pm (for display on map):
function localtime(time) {
	var hours = new Date(time).getHours();
	var mins = new Date(time).getMinutes();
	if (mins == 0) {
		mins = mins + '0';
	}
	if (hours == 0) {
		return "12:" + mins + " am";
	}
	else if (hours > 21) {
		hours = "1" + (hours-2).toString().substring(1);
		return hours + ":" + mins + " pm";
	}
	else if (hours == 12) {
		return hours + ":" + mins + " pm";
	}
	else if (hours > 12) {
		hours = (hours-2).toString().substring(1);
		return hours + ":" + mins + " pm";
	} else {
		return hours + ":" + mins + " am";
	}
}

// we must define the geographic bounds of the data we request from environment canada WMS.
// also need to transform the coordinates from our map projection (simple lat and lon) to the projection expected by WMS
// at least until i figure out how to specify in the request..?
// this should roughly correspond to the north eastern patch of north america
var x1 = -95, y1 = 38, x2 = -69, y2 = 50;
var ecExtent = ol.proj.transform([x1, y1, x2, y2], 'EPSG:4326', 'EPSG:3857');

// now lets define the layers of our map
// background is a nice monochrome map from stamen design
var layers = [

new ol.layer.Tile({
	preload: 2, // think preload applies to tiles on same zoom or lower (ie, panning and zooming out)
	source: new ol.source.XYZ({
		url: 'http://api.tiles.mapbox.com/v4/modzu.lbkb6pda/{z}/{x}/{y}.png?access_token=pk.eyJ1IjoibW9kenUiLCJhIjoiM1d3cjYxWSJ9.83JMa0PxznRkq6tSnWFqtQ'
		})
}),

// radar layer from environment canaduh
new ol.layer.Image({
	opacity: 0.5,
	extent: ecExtent,
	source: new ol.source.ImageWMS({
		ratio: 1,
		url: 'http://geo.weather.gc.ca/geomet/',
		params: {'LAYERS': 'RADAR_RRAI', 'time': maptime},
		// lets give credit where credit is due
		attributions: [
			new ol.Attribution({html: '<br>Meteorological data provided by <a href="http://www.ec.gc.ca">Environment Canada</a>'}),
			ol.source.OSM.DATA_ATTRIBUTION
		]
	})
}),

// previous hour radar layer
new ol.layer.Image({
	opacity: 0,
	extent: ecExtent,
	source: new ol.source.ImageWMS({
		ratio: 1,
		url: 'http://geo.weather.gc.ca/geomet/',
		params: {'LAYERS': 'RADAR_RRAI', 'time': newMaptime(maptime, 1)}
	})
}),

// 2 hours previous radar
new ol.layer.Image({
	opacity: 0,
	extent: ecExtent,
	source: new ol.source.ImageWMS({
		ratio: 1,
		url: 'http://geo.weather.gc.ca/geomet/',
		params: {'LAYERS': 'RADAR_RRAI', 'time': newMaptime(maptime, 2)}
	})
}),

// 3 hours previous radar
new ol.layer.Image({
	opacity: 0,
	extent: ecExtent,
	source: new ol.source.ImageWMS({
		ratio: 1,
		url: 'http://geo.weather.gc.ca/geomet/',
		params: {'LAYERS': 'RADAR_RRAI', 'time': newMaptime(maptime, 3)}
	})
})

];

/*
// future hour radar layer
var radarNext = new ol.layer.Image({
	opacity: 0,
	extent: ecExtent,
	source: new ol.source.ImageWMS({
		url: 'http://geo.weather.gc.ca/geomet/',
		params: {'LAYERS': 'HRDPS.EASTETA_PR', 'time': maptime, 'STYLES': 'CAPA24'},
	})
});
*/

// lets do a basic ghetto check for a 404 from environment canada - if the wms doesnt return an image, we can
// display an error message. this assumes toronto and default zoom level..

var wmsTestUrl = 'http://geo.weather.gc.ca/geomet/?SERVICE=WMS&VERSION=1.3.0&REQUEST=GetMap&FORMAT=image%2Fpng&TRANSPARENT=true&LAYERS=RADAR_RRAI&time=' + escape(newMaptime(maptime, 1)) + '&CRS=EPSG%3A3857&STYLES=&WIDTH=1535&HEIGHT=1038&BBOX=-10809848.224479826%2C4155601.1292604646%2C-7055261.395111969%2C6694533.460780879';

$(".imgtest").attr("src", wmsTestUrl);

$('.imgtest').on('error',function() {
	alert("Error connecting to Environment Canada WMS Server. Radar data may not be available at the present time.");
});


// geolocation
function getLocation() {
    if (navigator.geolocation) {
        navigator.geolocation.getCurrentPosition(showPosition, positionError, {timeout: 4000});
    } else {
		showPosition();
		$(".timedisplay").html("Geolocation unavailable");
    }
}

function showPosition(position) {
    //var lat = position.coords.latitude || 43.6479267;
	var lat = (position) ? position.coords.latitude : 43.6479267;
	//var lon = position.coords.longitude || -79.4197497;
	var lon = (position) ? position.coords.longitude : -79.4197497;
	// lets create the map (and center on our position)
	// also accidental pinch rotate was annoying me
	var ourLocation = ol.proj.transform([lon, lat], 'EPSG:4326', 'EPSG:3857');
	var interactions = ol.interaction.defaults({altShiftDragRotate:false, pinchRotate:false});
	var map = new ol.Map({
		interactions: interactions,
		layers: layers,
		target: 'map',
		view: new ol.View({
			center: ourLocation,
			zoom: 6,
			minZoom: 4
		})
	});
	// map is rendering - remove the loading message
	// could do better since we dont know when the layers are done downloading.. for now adding a small delay)
	$(".timedisplay").delay(1000).fadeOut(1000);
}

// geolocation error handler
// more elegant solution would be to use some default coordinates when location unavailable, but im too lazy
function positionError(error) {
    switch(error.code) {
        case error.PERMISSION_DENIED:
			$(".timedisplay").html("Location permissions denied.");
			showPosition();
            break;
        case error.POSITION_UNAVAILABLE:
			$(".timedisplay").html("Location information is currently unavailable.");
			showPosition();
            break;
        case error.TIMEOUT:
		   $(".timedisplay").html("Location request timed out.");
		   showPosition();
            break;
        case error.UNKNOWN_ERROR:
			$(".timedisplay").html("An unknown error occurred.");
			showPosition();
            break;
    }
}


// back and forward time nav buttons
// when the button is clicked: unhide the next layer and hide the previous layer, display the time of the new layer, and update our layer index to keep track of where we are

$(".back").click(function() {
	if(layerIndex<4) {
	$(".forward").removeClass("disabled");
	layers[layerIndex].setOpacity(0);
	layers[layerIndex+1].setOpacity(0.5);
	$(".timedisplay").html(localtime(newMaptime(maptime, layerIndex)))
	$(".timedisplay").fadeIn();
	layerIndex++;
	display = 1;
	lastclick = new Date();
	if (layerIndex == 4) {
		$(".back").addClass("disabled");
	}
	}
});

$(".forward").click(function() {
	if(layerIndex>1) {
	$(".back").removeClass("disabled");
	layers[layerIndex-1].setOpacity(0.5);
	layers[layerIndex].setOpacity(0);
	$(".timedisplay").html(localtime(newMaptime(maptime, layerIndex-2)))
	$(".timedisplay").fadeIn();
	layerIndex--;
	display = 1;
	lastclick = new Date();
	if (layerIndex == 1) {
		$(".forward").addClass("disabled");
	}
	}
});


// ENGAGE!!!
getLocation();

// there is a legend graphic not doing shit. will maybe add this in later somewhere
//$(".legend").show();

/*
other layers available from STAMEN:
toner, toner-hybrid, toner-labels, toner-lines, toner-background, toner-lite, terrain, terrain-labels, terrain-lines, terrain-background, watercolor, watercolor-
*/
